@@ -1,4 +1,22 @@
 # -*- coding: utf-8 -*-
+
+# Copyright © 2011-2014, 2016 marmuta <marmvta@gmail.com>
+#
+# This file is part of Onboard.
+#
+# Onboard is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
+#
+# Onboard is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
+
 """
 Module for theme related classes.
 """
@@ -456,13 +474,9 @@ class ColorScheme(object):
         """
 
         if state is None:
-            state = {}
-            state["prelight"]    =  key.prelight
-            state["pressed"]     =  key.pressed
-            state["active"]      =  key.active
-            state["locked"]      =  key.locked
-            state["scanned"]     =  key.scanned
-            state["insensitive"] =  not key.sensitive
+            state = key.get_state()
+            state["insensitive"] = not key.sensitive
+            del state["sensitive"]
 
         rgb = None
         opacity = None
